@@ -198,6 +198,19 @@ class FrameError(Exception):
     pass
 
 
+def _check_widget_subclass(widget: Widget | None) -> None:
+    if widget is None:
+        return
+
+    if not isinstance(widget, Widget):
+        obj_class_path = f"{widget.__class__.__module__}.{widget.__class__.__name__}"
+        warnings.warn(
+            f"{obj_class_path} is not subclass of Widget",
+            DeprecationWarning,
+            stacklevel=3,
+        )
+
+
 class Frame(Widget, WidgetContainerMixin):
     """
     Frame widget is a box widget with optional header and footer
@@ -235,6 +248,9 @@ class Frame(Widget, WidgetContainerMixin):
         self._body = body
         self._footer = footer
         self.focus_part = focus_part
+        _check_widget_subclass(header)
+        _check_widget_subclass(body)
+        _check_widget_subclass(footer)
 
     @property
     def header(self) -> Widget | None:
@@ -242,6 +258,7 @@ class Frame(Widget, WidgetContainerMixin):
 
     @header.setter
     def header(self, header: Widget | None):
+        _check_widget_subclass(header)
         self._header = header
         if header is None and self.focus_part == "header":
             self.focus_part = "body"
@@ -271,6 +288,7 @@ class Frame(Widget, WidgetContainerMixin):
 
     @body.setter
     def body(self, body: Widget) -> None:
+        _check_widget_subclass(body)
         self._body = body
         self._invalidate()
 
@@ -298,6 +316,7 @@ class Frame(Widget, WidgetContainerMixin):
 
     @footer.setter
     def footer(self, footer: Widget | None) -> None:
+        _check_widget_subclass(footer)
         self._footer = footer
         if footer is None and self.focus_part == "footer":
             self.focus_part = "body"
@@ -340,7 +359,7 @@ class Frame(Widget, WidgetContainerMixin):
         :param part: 'header', 'footer' or 'body'
         :type part: str
         """
-        if part not in ("header", "footer", "body"):
+        if part not in {"header", "footer", "body"}:
             raise IndexError(f"Invalid position for Frame: {part}")
         if (part == "header" and self._header is None) or (part == "footer" and self._footer is None):
             raise IndexError(f"This Frame has no {part}")
@@ -461,7 +480,7 @@ class Frame(Widget, WidgetContainerMixin):
         raise KeyError(f"Frame.contents has no key: {key!r}")
 
     def _contents__setitem__(self, key: Literal["header", "footer", "body"], value):
-        if key not in ("body", "header", "footer"):
+        if key not in {"body", "header", "footer"}:
             raise KeyError(f"Frame.contents has no key: {key!r}")
         try:
             value_w, value_options = value
@@ -477,7 +496,7 @@ class Frame(Widget, WidgetContainerMixin):
             self.header = value_w
 
     def _contents__delitem__(self, key: Literal["header", "footer", "body"]):
-        if key not in ("header", "footer"):
+        if key not in {"header", "footer"}:
             raise KeyError(f"Frame.contents can't remove key: {key!r}")
         if (key == "header" and self._header is None) or (key == "footer" and self._footer is None):
             raise KeyError(f"Frame.contents has no key: {key!r}")
@@ -501,7 +520,6 @@ class Frame(Widget, WidgetContainerMixin):
 
         Return None as a placeholder for future options.
         """
-        return
 
     def frame_top_bottom(self, size: tuple[int, int], focus: bool) -> tuple[tuple[int, int], tuple[int, int]]:
         """
@@ -622,7 +640,7 @@ class Frame(Widget, WidgetContainerMixin):
         Focus may be changed on button 1 press.
         """
         (maxcol, maxrow) = size
-        (htrim, ftrim), (hrows, frows) = self.frame_top_bottom((maxcol, maxrow), focus)
+        (htrim, ftrim), (_hrows, _frows) = self.frame_top_bottom((maxcol, maxrow), focus)
 
         if row < htrim:  # within header
             focus = focus and self.focus_part == "header"
